@@ -7,6 +7,7 @@ package scan
 import (
 	"fmt"
 	"io"
+	"sort"
 	"strings"
 
 	"golang.org/x/vuln/internal"
@@ -40,15 +41,24 @@ type TextHandler struct {
 
 	err error
 
-	showColor   bool
-	showTraces  bool
-	showVersion bool
+	showColor    bool
+	showTraces   bool
+	showVersion  bool
+	showAllVulns bool
 }
 
 const (
 	detailsMessage = `For details, see https://pkg.go.dev/golang.org/x/vuln/cmd/govulncheck.`
 
 	binaryProgressMessage = `Scanning your binary for known vulnerabilities...`
+
+	noVulnsMessage = `No vulnerabilities found.`
+
+	noOtherVulnsMessage = `No other vulnerabilities found.`
+
+	verboseMessage = `'-show verbose' for more details`
+
+	symbolMessage = `'-scan symbol' for more fine grained vulnerability detection`
 )
 
 func (h *TextHandler) Show(show []string) {
@@ -60,28 +70,30 @@ func (h *TextHandler) Show(show []string) {
 			h.showColor = true
 		case "version":
 			h.showVersion = true
+		case "verbose":
+			h.showAllVulns = true
 		}
 	}
 }
 
-func Flush(h govulncheck.Handler) error {
-	if th, ok := h.(interface{ Flush() error }); ok {
-		return th.Flush()
-	}
-	return nil
-}
-
 func (h *TextHandler) Flush() error {
-	fixupFindings(h.osvs, h.findings)
-	h.byVulnerability(h.findings)
-	h.summary(h.findings)
-	h.print("Share feedback at https://go.dev/s/govulncheck-feedback.\n")
+	if len(h.findings) == 0 {
+		h.print(noVulnsMessage + "\n")
+	} else {
+		fixupFindings(h.osvs, h.findings)
+		counters := h.allVulns(h.findings)
+		h.summary(counters)
+	}
 	if h.err != nil {
 		return h.err
 	}
-	if isCalled(h.findings) {
+	// We found vulnerabilities when the findings' level matches the scan level.
+	if (isCalled(h.findings) && h.scanLevel == govulncheck.ScanLevelSymbol) ||
+		(isImported(h.findings) && h.scanLevel == govulncheck.ScanLevelPackage) ||
+		(isRequired(h.findings) && h.scanLevel == govulncheck.ScanLevelModule) {
 		return errVulnerabilitiesFound
 	}
+
 	return nil
 }
 
@@ -117,7 +129,7 @@ func (h *TextHandler) Config(config *govulncheck.Config) error {
 	return h.err
 }
 
-// Progress writes progress updates during govulncheck execution..
+// Progress writes progress updates during govulncheck execution.
 func (h *TextHandler) Progress(progress *govulncheck.Progress) error {
 	h.print(progress.Message, "\n\n")
 	return h.err
@@ -138,63 +150,63 @@ func (h *TextHandler) Finding(finding *govulncheck.Finding) error {
 	return nil
 }
 
-func (h *TextHandler) byVulnerability(findings []*findingSummary) {
+func (h *TextHandler) allVulns(findings []*findingSummary) summaryCounters {
 	byVuln := groupByVuln(findings)
-	called := 0
-	onlyImported := 0
+	var called, imported, required [][]*findingSummary
+	mods := map[string]struct{}{}
+	stdlibCalled := false
 	for _, findings := range byVuln {
-		if isCalled(findings) {
-			h.vulnerability(called, findings)
-			called++
-		} else if isImported(findings) && !isStdFindings(findings) {
-			onlyImported++
+		switch {
+		case isCalled(findings):
+			called = append(called, findings)
+			if isStdFindings(findings) {
+				stdlibCalled = true
+			} else {
+				mods[findings[0].Trace[0].Module] = struct{}{}
+			}
+		case isImported(findings):
+			imported = append(imported, findings)
+		default:
+			required = append(required, findings)
 		}
 	}
-	onlyRequired := len(byVuln) - (called + onlyImported)
-	if onlyImported+onlyRequired == 0 {
-		return
-	}
-	if h.scanLevel.WantSymbols() {
-		h.style(sectionStyle, "=== Informational ===\n\n")
-	}
-	var informational strings.Builder
-	if onlyImported > 0 {
-		informational.WriteString("Found " + fmt.Sprint(onlyImported))
-		informational.WriteString(choose(onlyImported == 1, ` vulnerability`, ` vulnerabilities`))
-		informational.WriteString(" in packages that you import")
-		if h.scanLevel.WantSymbols() {
-			informational.WriteString(", but there are no call stacks leading to the use of ")
-			informational.WriteString(choose(onlyImported == 1, `this vulnerability.`, `these vulnerabilities.`))
-		} else {
-			informational.WriteString(".")
-		}
-	}
-	if onlyRequired > 0 {
-		isare := choose(onlyRequired == 1, ` is `, ` are `)
-		informational.WriteString(" There" + isare + choose(onlyImported > 0, `also `, ``) + fmt.Sprint(onlyRequired))
-		informational.WriteString(choose(onlyRequired == 1, ` vulnerability `, ` vulnerabilities `))
-		informational.WriteString("in modules that you require")
-		if h.scanLevel.WantSymbols() {
-			informational.WriteString(" that" + choose(h.scanLevel.WantSymbols(), isare, " may be "))
-			informational.WriteString("neither imported nor called.")
-		} else {
-			informational.WriteString(".")
-		}
 
-	}
 	if h.scanLevel.WantSymbols() {
-		informational.WriteString(" You may not need to take any action.")
-	} else {
-		informational.WriteString(" Use -scan=symbol with govulncheck for more fine grained vulnerability detection.")
-	}
-	h.wrap("", informational.String(), 70)
-	h.print("\nSee https://pkg.go.dev/golang.org/x/vuln/cmd/govulncheck for details.\n\n")
-	index := 0
-	for _, findings := range byVuln {
-		if !isCalled(findings) {
-			h.vulnerability(index, findings)
-			index++
+		h.style(sectionStyle, "=== Symbol Results ===\n\n")
+		if len(called) == 0 {
+			h.print(noVulnsMessage, "\n\n")
 		}
+		for index, findings := range called {
+			h.vulnerability(index, findings)
+		}
+	}
+
+	if h.scanLevel == govulncheck.ScanLevelPackage || (h.scanLevel.WantPackages() && h.showAllVulns) {
+		h.style(sectionStyle, "=== Package Results ===\n\n")
+		if len(imported) == 0 {
+			h.print(choose(!h.scanLevel.WantSymbols(), noVulnsMessage, noOtherVulnsMessage), "\n\n")
+		}
+		for index, findings := range imported {
+			h.vulnerability(index, findings)
+		}
+	}
+
+	if h.showAllVulns || h.scanLevel == govulncheck.ScanLevelModule {
+		h.style(sectionStyle, "=== Module Results ===\n\n")
+		if len(required) == 0 {
+			h.print(choose(!h.scanLevel.WantPackages(), noVulnsMessage, noOtherVulnsMessage), "\n\n")
+		}
+		for index, findings := range required {
+			h.vulnerability(index, findings)
+		}
+	}
+
+	return summaryCounters{
+		VulnerabilitiesCalled:   len(called),
+		VulnerabilitiesImported: len(imported),
+		VulnerabilitiesRequired: len(required),
+		ModulesCalled:           len(mods),
+		StdlibCalled:            stdlibCalled,
 	}
 }
 
@@ -221,13 +233,20 @@ func (h *TextHandler) vulnerability(index int, findings []*findingSummary) {
 	byModule := groupByModule(findings)
 	first := true
 	for _, module := range byModule {
-		//TODO: this assumes all traces on a module are found and fixed at the same versions
+		// Note: there can be several findingSummaries for the same vulnerability
+		// emitted during streaming for different scan levels.
+
+		// The module is same for all finding summaries.
 		lastFrame := module[0].Trace[0]
 		mod := lastFrame.Module
+		// For stdlib, try to show package path as module name where
+		// the scan level allows it.
+		// TODO: should this be done in byModule as well?
 		path := lastFrame.Module
-		if path == internal.GoStdModulePath {
-			path = lastFrame.Package
+		if stdPkg := h.pkg(module); path == internal.GoStdModulePath && stdPkg != "" {
+			path = stdPkg
 		}
+		// All findings on a module are found and fixed at the same version
 		foundVersion := moduleVersionString(lastFrame.Module, lastFrame.Version)
 		fixedVersion := moduleVersionString(lastFrame.Module, module[0].FixedVersion)
 		if !first {
@@ -267,7 +286,29 @@ func (h *TextHandler) vulnerability(index int, findings []*findingSummary) {
 	h.print("\n")
 }
 
+// pkg gives the package information for findings summaries
+// if one exists. This is only used to print package path
+// instead of a module for stdlib vulnerabilities at symbol
+// and package scan level.
+func (h *TextHandler) pkg(summaries []*findingSummary) string {
+	for _, f := range summaries {
+		if pkg := f.Trace[0].Package; pkg != "" {
+			return pkg
+		}
+	}
+	return ""
+}
+
+// traces prints out the most precise trace information
+// found in the given summaries.
 func (h *TextHandler) traces(traces []*findingSummary) {
+	// Sort the traces by the vulnerable symbol. This
+	// guarantees determinism since we are currently
+	// showing only one trace per symbol.
+	sort.SliceStable(traces, func(i, j int) bool {
+		return symbol(traces[i].Trace[0], true) < symbol(traces[j].Trace[0], true)
+	})
+
 	first := true
 	count := 1
 	for _, entry := range traces {
@@ -297,28 +338,90 @@ func (h *TextHandler) traces(traces []*findingSummary) {
 	}
 }
 
-func (h *TextHandler) summary(findings []*findingSummary) {
-	counters := counters(findings)
-	if counters.VulnerabilitiesCalled == 0 {
-		h.print(choose(h.scanLevel.WantSymbols(), "No vulnerabilities found.\n\n", ""))
-		return
+func (h *TextHandler) summary(c summaryCounters) {
+	// print short summary of findings identified at the desired level of scan precision
+	var vulnCount int
+	h.print("Your code ", choose(h.scanLevel.WantSymbols(), "is", "may be"), " affected by ")
+	switch h.scanLevel {
+	case govulncheck.ScanLevelSymbol:
+		vulnCount = c.VulnerabilitiesCalled
+	case govulncheck.ScanLevelPackage:
+		vulnCount = c.VulnerabilitiesImported
+	case govulncheck.ScanLevelModule:
+		vulnCount = c.VulnerabilitiesRequired
 	}
-	h.print(`Your code is affected by `)
-	h.style(valueStyle, counters.VulnerabilitiesCalled)
-	h.print(choose(counters.VulnerabilitiesCalled == 1, ` vulnerability`, ` vulnerabilities`))
-	h.print(` from`)
-	if counters.ModulesCalled > 0 {
-		h.print(` `)
-		h.style(valueStyle, counters.ModulesCalled)
-		h.print(choose(counters.ModulesCalled == 1, ` module`, ` modules`))
-	}
-	if counters.StdlibCalled {
-		if counters.ModulesCalled != 0 {
-			h.print(` and`)
+	h.style(valueStyle, vulnCount)
+	h.print(choose(vulnCount == 1, ` vulnerability`, ` vulnerabilities`))
+	if h.scanLevel.WantSymbols() {
+		h.print(choose(c.ModulesCalled > 0 || c.StdlibCalled, ` from `, ``))
+		if c.ModulesCalled > 0 {
+			h.style(valueStyle, c.ModulesCalled)
+			h.print(choose(c.ModulesCalled == 1, ` module`, ` modules`))
+		}
+		if c.StdlibCalled {
+			if c.ModulesCalled != 0 {
+				h.print(` and `)
+			}
+			h.print(`the Go standard library`)
 		}
-		h.print(` the Go standard library`)
 	}
-	h.print(".\n\n")
+	h.print(".\n")
+
+	// print summary for vulnerabilities found at other levels of scan precision
+	if other := h.summaryOtherVulns(c); other != "" {
+		h.wrap("", other, 80)
+		h.print("\n")
+	}
+
+	// print suggested flags for more/better info depending on scan level and if in verbose mode
+	if sugg := h.summarySuggestion(); sugg != "" {
+		h.wrap("", sugg, 80)
+		h.print("\n")
+	}
+}
+
+func (h *TextHandler) summaryOtherVulns(c summaryCounters) string {
+	var summary strings.Builder
+	if c.VulnerabilitiesRequired+c.VulnerabilitiesImported == 0 {
+		summary.WriteString("This scan found no other vulnerabilities in ")
+		if h.scanLevel.WantSymbols() {
+			summary.WriteString("packages you import or ")
+		}
+		summary.WriteString("modules you require.")
+	} else {
+		summary.WriteString(choose(h.scanLevel.WantPackages(), "This scan also found ", ""))
+		if h.scanLevel.WantSymbols() {
+			summary.WriteString(fmt.Sprint(c.VulnerabilitiesImported))
+			summary.WriteString(choose(c.VulnerabilitiesImported == 1, ` vulnerability `, ` vulnerabilities `))
+			summary.WriteString("in packages you import and ")
+		}
+		if h.scanLevel.WantPackages() {
+			summary.WriteString(fmt.Sprint(c.VulnerabilitiesRequired))
+			summary.WriteString(choose(c.VulnerabilitiesRequired == 1, ` vulnerability `, ` vulnerabilities `))
+			summary.WriteString("in modules you require")
+			summary.WriteString(choose(h.scanLevel.WantSymbols(), ", but your code doesn't appear to call these vulnerabilities.", "."))
+		}
+	}
+	return summary.String()
+}
+
+func (h *TextHandler) summarySuggestion() string {
+	var sugg strings.Builder
+	switch h.scanLevel {
+	case govulncheck.ScanLevelSymbol:
+		if !h.showAllVulns {
+			sugg.WriteString("Use " + verboseMessage + ".")
+		}
+	case govulncheck.ScanLevelPackage:
+		sugg.WriteString("Use " + symbolMessage)
+		if !h.showAllVulns {
+			sugg.WriteString(" and " + verboseMessage)
+		}
+		sugg.WriteString(".")
+	case govulncheck.ScanLevelModule:
+		sugg.WriteString("Use " + symbolMessage + ".")
+	}
+	return sugg.String()
 }
 
 func (h *TextHandler) style(style style, values ...any) {
@@ -390,7 +493,7 @@ func choose[t any](b bool, yes, no t) t {
 
 func isStdFindings(findings []*findingSummary) bool {
 	for _, f := range findings {
-		if vulncheck.IsStdPackage(f.Trace[0].Package) {
+		if vulncheck.IsStdPackage(f.Trace[0].Package) || f.Trace[0].Module == internal.GoStdModulePath {
 			return true
 		}
 	}
